@@ -28,7 +28,7 @@ sub create_drop_or_attach {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $old_idx_cda = 1;
+    state $old_idx_cda = 1;
 
     CREATE_DROP_ATTACH: while ( 1 ) {
         my $hidden = $sf->{d}{db_string};
@@ -74,35 +74,43 @@ sub create_drop_or_attach {
         elsif ( $choice =~ /^-\ Create/i ) {
             require App::DBBrowser::CreateDropAttach::CreateTable;
             my $ct = App::DBBrowser::CreateDropAttach::CreateTable->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $changed;
             if ( $choice eq $create_table ) {
-                if ( ! eval { $ct->create_table(); 1 } ) {
+                if ( ! eval { $changed = $ct->create_table(); 1 } ) {
                     $ax->print_error_message( $@ );
                 }
             }
             elsif ( $choice eq $create_view ) {
-                if ( ! eval { $ct->create_view(); 1 } ) {
+                if ( ! eval { $changed = $ct->create_view(); 1 } ) {
                     $ax->print_error_message( $@ );
                 }
             }
+            if ( $changed ) {
+                return 1;
+            }
         }
         elsif ( $choice =~ /^-\ Drop/i ) {
             require App::DBBrowser::CreateDropAttach::DropTable;
             my $dt = App::DBBrowser::CreateDropAttach::DropTable->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $changed;
             if ( $choice eq $drop_table ) {
-                if ( ! eval { $dt->drop_table(); 1 } ) {
+                if ( ! eval { $changed = $dt->drop_table(); 1 } ) {
                     $ax->print_error_message( $@ );
                 }
             }
             elsif ( $choice eq $drop_view ) {
-                if ( ! eval { $dt->drop_view(); 1 } ) {
+                if ( ! eval { $changed = $dt->drop_view(); 1 } ) {
                     $ax->print_error_message( $@ );
                 }
             }
+            if ( $changed ) {
+                return 1;
+            }
         }
         elsif ( $choice =~ /^-\ (?:Attach|Detach)/ ) {
             require App::DBBrowser::CreateDropAttach::AttachDB;
             my $att = App::DBBrowser::CreateDropAttach::AttachDB->new( $sf->{i}, $sf->{o}, $sf->{d} );
-            my $changed; ##
+            my $changed;
             if ( $choice eq $attach_databases ) {
                 if ( ! eval { $changed = $att->attach_db(); 1 } ) {
                     $ax->print_error_message( $@ );
@@ -113,16 +121,9 @@ sub create_drop_or_attach {
                     $ax->print_error_message( $@ );
                 }
             }
-        }
-        if ( ! eval {
-            my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
-            ( $sf->{d}{tables_info},
-              $sf->{d}{user_table_keys},
-              $sf->{d}{sys_table_keys} ) = $plui->tables_info( $sf->{d}{dbh}, $sf->{d}{schema},
-                                                               $sf->{d}{is_system_schema}, $sf->{d}{db_attached} );
-            1 }
-         ) {
-            $ax->print_error_message( $@ );
+            if ( $changed ) {
+                return 2;
+            }
         }
     }
 }
