@@ -99,35 +99,22 @@ sub get_db_handle {
 sub get_databases {
     my ( $sf ) = @_;
     return \@ARGV if @ARGV;
-    my @regex_system_db = ( '^mysql$', '^information_schema$', '^performance_schema$', '^sys$' );
-    my $stmt = "SELECT schema_name FROM information_schema.schemata";
-    if ( ! $sf->{o}{G}{metadata} ) {
-        $stmt .= " WHERE " . join( " AND ", ( "schema_name NOT REGEXP ?" ) x @regex_system_db );
-    }
-    $stmt .= " ORDER BY schema_name";
     my $info_database = 'information_schema';
-    #print $sf->{clear_screen};
-    #print "DB: $info_database\n";
     my $dbh = $sf->get_db_handle( $info_database );
-    my $databases = $dbh->selectcol_arrayref( $stmt, {}, $sf->{o}{G}{metadata} ? () : @regex_system_db );
+    my $stmt = "SELECT schema_name FROM information_schema.schemata ORDER BY schema_name";
+    my $databases = $dbh->selectcol_arrayref( $stmt, {}, () );
     $dbh->disconnect();
-    if ( $sf->{o}{G}{metadata} ) {
-        my $regexp = join '|', @regex_system_db;
-        my $user_db   = [];
-        my $system_db = [];
-        for my $database ( @{$databases} ) {
-            if ( $database =~ /(?:$regexp)/ ) {
-                push @$system_db, $database;
-            }
-            else {
-                push @$user_db, $database;
-            }
+    my $user_db   = [];
+    my $system_db = [];
+    for my $database ( @{$databases} ) {
+        if ( $database =~ /^(?:mysql|information_schema|performance_schema|sys)\z/ ) {
+            push @$system_db, $database;
+        }
+        else {
+            push @$user_db, $database;
         }
-        return $user_db, $system_db;
-    }
-    else {
-        return $databases;
     }
+    return $user_db, $system_db;
 }
 
 
@@ -135,10 +122,6 @@ sub get_databases {
 
 
 
-
-
-
-
 1;
 
 
